@@ -36,7 +36,7 @@
 //! **declarations**, one for the `fill` **property** and one for the
 //! `stroke` property.
 //!
-//! After the first qualified rule, we have a second qualified rule with
+//! After ther first qualified rule, we have a second qualified rule with
 //! a single selector for the `#baz` id, with a single declaration for the
 //! `stroke-width` property.
 //!
@@ -78,23 +78,20 @@ use cssparser::{
     CowRcStr, DeclarationListParser, DeclarationParser, Parser, ParserInput, QualifiedRuleParser,
     RuleListParser, SourceLocation, ToCss, _cssparser_internal_to_lowercase,
 };
+use markup5ever::{namespace_url, ns, LocalName, Namespace, Prefix, QualName};
 use selectors::attr::{AttrSelectorOperation, CaseSensitivity, NamespaceConstraint};
 use selectors::matching::{ElementSelectorFlags, MatchingContext, MatchingMode, QuirksMode};
 use selectors::{OpaqueElement, SelectorImpl, SelectorList};
-
 use std::cmp::Ordering;
 use std::fmt;
 use std::str;
-
-use markup5ever::{namespace_url, ns, LocalName, Namespace, Prefix, QualName};
 use url::Url;
 
 use crate::allowed_url::AllowedUrl;
 use crate::error::*;
 use crate::io::{self, BinaryData};
-use crate::node::{NodeCascade, NodeType, RsvgNode};
+use crate::node::{Node, NodeBorrow, NodeCascade};
 use crate::properties::{parse_property, ComputedValues, ParsedProperty};
-use crate::text::NodeChars;
 
 /// A parsed CSS declaration
 ///
@@ -364,17 +361,17 @@ impl SelectorImpl for Selector {
     type PseudoElement = PseudoElement;
 }
 
-/// Wraps an `RsvgNode` with a locally-defined type, so we can implement
+/// Wraps an `Node` with a locally-defined type, so we can implement
 /// a foreign trait on it.
 ///
-/// RsvgNode is an alias for rctree::Node, so we can't implement
+/// `Node` is an alias for `rctree::Node`, so we can't implement
 /// `selectors::Element` directly on it.  We implement it on the
 /// `RsvgElement` wrapper instead.
 #[derive(Clone, PartialEq)]
-pub struct RsvgElement(RsvgNode);
+pub struct RsvgElement(Node);
 
-impl From<RsvgNode> for RsvgElement {
-    fn from(n: RsvgNode) -> RsvgElement {
+impl From<Node> for RsvgElement {
+    fn from(n: Node) -> RsvgElement {
         RsvgElement(n)
     }
 }
@@ -421,7 +418,7 @@ impl selectors::Element for RsvgElement {
         let mut sibling = self.0.previous_sibling();
 
         while let Some(ref sib) = sibling {
-            if sib.borrow().get_type() != NodeType::Chars {
+            if sib.is_element() {
                 return sibling.map(|n| n.into());
             }
 
@@ -436,7 +433,7 @@ impl selectors::Element for RsvgElement {
         let mut sibling = self.0.next_sibling();
 
         while let Some(ref sib) = sibling {
-            if sib.borrow().get_type() != NodeType::Chars {
+            if sib.is_element() {
                 return sibling.map(|n| n.into());
             }
 
@@ -451,17 +448,17 @@ impl selectors::Element for RsvgElement {
     }
 
     fn has_local_name(&self, local_name: &LocalName) -> bool {
-        self.0.borrow().element_name().local == *local_name
+        self.0.borrow_element().element_name().local == *local_name
     }
 
     /// Empty string for no namespace
     fn has_namespace(&self, ns: &Namespace) -> bool {
-        self.0.borrow().element_name().ns == *ns
+        self.0.borrow_element().element_name().ns == *ns
     }
 
     /// Whether this element and the `other` element have the same local name and namespace.
     fn is_same_type(&self, other: &Self) -> bool {
-        self.0.borrow().element_name() == other.0.borrow().element_name()
+        self.0.borrow_element().element_name() == other.0.borrow_element().element_name()
     }
 
     fn attr_matches(
@@ -499,7 +496,7 @@ impl selectors::Element for RsvgElement {
     /// Whether this element is a `link`.
     fn is_link(&self) -> bool {
         // FIXME: is this correct for SVG <a>, not HTML <a>?
-        self.0.borrow().get_type() == NodeType::Link
+        self.0.is_element() && is_element_of_type!(self.0, Link)
     }
 
     /// Returns whether the element is an HTML <slot> element.
@@ -509,7 +506,7 @@ impl selectors::Element for RsvgElement {
 
     fn has_id(&self, id: &LocalName, case_sensitivity: CaseSensitivity) -> bool {
         self.0
-            .borrow()
+            .borrow_element()
             .get_id()
             .map(|self_id| case_sensitivity.eq(self_id.as_bytes(), id.as_ref().as_bytes()))
             .unwrap_or(false)
@@ -517,7 +514,7 @@ impl selectors::Element for RsvgElement {
 
     fn has_class(&self, name: &LocalName, case_sensitivity: CaseSensitivity) -> bool {
         self.0
-            .borrow()
+            .borrow_element()
             .get_class()
             .map(|classes| {
                 classes
@@ -547,11 +544,10 @@ impl selectors::Element for RsvgElement {
     /// That is, whether it does not contain any child element or any non-zero-length text node.
     /// See http://dev.w3.org/csswg/selectors-3/#empty-pseudo
     fn is_empty(&self) -> bool {
-        !self.0.has_children()
-            || self.0.children().all(|child| {
-                child.borrow().get_type() == NodeType::Chars
-                    && child.borrow().get_impl::<NodeChars>().is_empty()
-            })
+        // .all() returns true for the empty iterator
+        self.0
+            .children()
+            .all(|child| child.is_chars() && child.borrow_chars().is_empty())
     }
 
     /// Returns whether this element matches `:root`,
@@ -567,7 +563,6 @@ impl selectors::Element for RsvgElement {
 /// Origin for a stylesheet, per https://www.w3.org/TR/CSS22/cascade.html#cascading-order
 ///
 /// This is used when sorting selector matches according to their origin and specificity.
-#[allow(unused)]
 #[derive(Copy, Clone, Eq, Ord, PartialEq, PartialOrd)]
 pub enum Origin {
     UserAgent,
@@ -707,7 +702,7 @@ impl Stylesheet {
     /// Appends the style declarations that match a specified node to a given vector
     fn get_matches<'a>(
         &'a self,
-        node: &RsvgNode,
+        node: &Node,
         match_ctx: &mut MatchingContext<Selector>,
         acc: &mut Vec<Match<'a>>,
     ) {
@@ -736,8 +731,13 @@ impl Stylesheet {
 }
 
 /// Runs the CSS cascade on the specified tree from all the stylesheets
-pub fn cascade(root: &mut RsvgNode, stylesheets: &[Stylesheet], extra: &[Stylesheet]) {
-    for mut node in root.descendants() {
+pub fn cascade(
+    root: &mut Node,
+    ua_stylesheets: &[Stylesheet],
+    author_stylesheets: &[Stylesheet],
+    user_stylesheets: &[Stylesheet],
+) {
+    for mut node in root.descendants().filter(|n| n.is_element()) {
         let mut matches = Vec::new();
 
         let mut match_ctx = MatchingContext::new(
@@ -749,17 +749,22 @@ pub fn cascade(root: &mut RsvgNode, stylesheets: &[Stylesheet], extra: &[Stylesh
             QuirksMode::NoQuirks,
         );
 
-        for stylesheet in stylesheets.iter().chain(extra.iter()) {
-            stylesheet.get_matches(&node, &mut match_ctx, &mut matches);
+        for s in ua_stylesheets
+            .iter()
+            .chain(author_stylesheets)
+            .chain(user_stylesheets)
+        {
+            s.get_matches(&node, &mut match_ctx, &mut matches);
         }
 
         matches.as_mut_slice().sort();
 
         for m in matches {
-            node.borrow_mut().apply_style_declaration(m.declaration);
+            node.borrow_element_mut()
+                .apply_style_declaration(m.declaration, m.origin);
         }
 
-        node.borrow_mut().set_style_attribute();
+        node.borrow_element_mut().set_style_attribute();
     }
 
     let values = ComputedValues::default();
@@ -815,11 +820,10 @@ mod tests {
             .unwrap();
 
         // Node types
-
-        assert!(a.borrow().get_type() == NodeType::Svg);
-        assert!(b.borrow().get_type() == NodeType::Rect);
-        assert!(c.borrow().get_type() == NodeType::Circle);
-        assert!(d.borrow().get_type() == NodeType::Rect);
+        assert!(is_element_of_type!(a, Svg));
+        assert!(is_element_of_type!(b, Rect));
+        assert!(is_element_of_type!(c, Circle));
+        assert!(is_element_of_type!(d, Rect));
 
         let a = RsvgElement(a);
         let b = RsvgElement(b);
