@@ -4,8 +4,9 @@ use crate::allowed_url::{Fragment, Href};
 use crate::aspect_ratio::AspectRatio;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
+use crate::element::{ElementResult, SetAttributes};
 use crate::error::*;
-use crate::node::{CascadedValues, NodeResult, NodeTrait, RsvgNode};
+use crate::node::{CascadedValues, Node};
 use crate::parsers::ParseValue;
 use crate::property_bag::PropertyBag;
 use crate::rect::Rect;
@@ -44,7 +45,7 @@ impl FeImage {
         fragment: &Fragment,
     ) -> Result<FilterResult, FilterError> {
         let acquired_drawable = acquired_nodes
-            .acquire(fragment, &[])
+            .acquire(fragment)
             .map_err(|_| FilterError::InvalidInput)?;
         let drawable = acquired_drawable.get();
 
@@ -86,7 +87,6 @@ impl FeImage {
             .lookup_image(url)
             .map_err(|_| FilterError::InvalidInput)?;
 
-        // TODO: this goes through a f64->i32->f64 conversion.
         let rect = self.aspect.compute(
             &ViewBox(Rect::from_size(
                 f64::from(image.width()),
@@ -109,11 +109,9 @@ impl FeImage {
     }
 }
 
-impl NodeTrait for FeImage {
-    impl_node_as_filter_effect!();
-
-    fn set_atts(&mut self, parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
-        self.base.set_atts(parent, pbag)?;
+impl SetAttributes for FeImage {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
+        self.base.set_attributes(pbag)?;
 
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
@@ -122,7 +120,7 @@ impl NodeTrait for FeImage {
                 // "path" is used by some older Adobe Illustrator versions
                 expanded_name!(xlink "href") | expanded_name!("", "path") => {
                     let href = Href::parse(value)
-                        .map_err(|e| ValueErrorKind::from(e))
+                        .map_err(ValueErrorKind::from)
                         .attribute(attr)?;
 
                     self.href = Some(href);
@@ -139,12 +137,12 @@ impl NodeTrait for FeImage {
 impl FilterEffect for FeImage {
     fn render(
         &self,
-        _node: &RsvgNode,
+        node: &Node,
         ctx: &FilterContext,
         acquired_nodes: &mut AcquiredNodes,
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
-        let bounds_builder = self.base.get_bounds(ctx);
+        let bounds_builder = self.base.get_bounds(ctx, node.parent().as_ref())?;
         let bounds = bounds_builder.into_rect(draw_ctx);
 
         match self.href.as_ref() {
