@@ -2,18 +2,17 @@
 //!
 //! This module provides the primitives on which the public APIs are implemented.
 
-use locale_config::{LanguageRange, Locale};
-
 use crate::allowed_url::{AllowedUrl, Href};
 use crate::bbox::BoundingBox;
 use crate::css::{Origin, Stylesheet};
 use crate::document::{AcquiredNodes, Document};
 use crate::dpi::Dpi;
 use crate::drawing_ctx::DrawingCtx;
+use crate::element::Element;
 use crate::error::{DefsLookupErrorKind, LoadingError, RenderingError};
-use crate::node::{CascadedValues, RsvgNode};
+use crate::node::{CascadedValues, Node, NodeBorrow};
 use crate::rect::Rect;
-use crate::structure::{IntrinsicDimensions, Svg};
+use crate::structure::IntrinsicDimensions;
 use url::Url;
 
 /// Loading options for SVG documents.
@@ -27,10 +26,6 @@ pub struct LoadOptions {
 
     /// Whether to keep original (undecoded) image data to embed in Cairo PDF surfaces.
     pub keep_image_data: bool,
-
-    /// The environment's locale, used for the `<switch>` element and the `systemLanguage`
-    /// attribute.
-    locale: Locale,
 }
 
 impl LoadOptions {
@@ -40,7 +35,6 @@ impl LoadOptions {
             base_url,
             unlimited_size: false,
             keep_image_data: false,
-            locale: locale_from_environment(),
         }
     }
 
@@ -71,20 +65,22 @@ impl LoadOptions {
             base_url: Some((**base_url).clone()),
             unlimited_size: self.unlimited_size,
             keep_image_data: self.keep_image_data,
-            locale: self.locale.clone(),
         }
     }
-
-    pub fn locale(&self) -> &Locale {
-        &self.locale
-    }
 }
 
+/// Main handle to an SVG document.
+///
+/// This is the main object in librsvg.  It gets created with the [`from_stream`] method
+/// and then provides access to all the primitives needed to implement the public APIs.
+///
+/// [`from_stream`]: #method.from_stream
 pub struct Handle {
     document: Document,
 }
 
 impl Handle {
+    /// Loads an SVG document into a `Handle`.
     pub fn from_stream(
         load_options: &LoadOptions,
         stream: &gio::InputStream,
@@ -95,6 +91,10 @@ impl Handle {
         })
     }
 
+    /// Queries whether a document has a certain element `#foo`.
+    ///
+    /// The `id` must be an URL fragment identifier, i.e. something
+    /// like `#element_id`.
     pub fn has_sub(&self, id: &str) -> Result<bool, RenderingError> {
         match self.lookup_node(id) {
             Ok(_) => Ok(true),
@@ -108,7 +108,7 @@ impl Handle {
     /// Returns (ink_rect, logical_rect)
     fn get_node_geometry_with_viewport(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         viewport: Rect,
         dpi: Dpi,
         is_testing: bool,
@@ -148,19 +148,21 @@ impl Handle {
             let cascaded = CascadedValues::new_from_node(&node);
             let values = cascaded.get();
 
-            if let Some((root_width, root_height)) =
-                node.borrow().get_impl::<Svg>().get_size(&values, dpi)
-            {
-                let rect = Rect::from_size(root_width, root_height);
-
-                return Ok((rect, rect));
+            match *node.borrow_element() {
+                Element::Svg(ref svg) => {
+                    if let Some((w, h)) = svg.get_size(&values, dpi) {
+                        let rect = Rect::from_size(w, h);
+                        return Ok((rect, rect));
+                    }
+                }
+                _ => (),
             }
         }
 
         self.get_node_geometry_with_viewport(&node, unit_rectangle(), dpi, is_testing)
     }
 
-    fn get_node_or_root(&self, id: Option<&str>) -> Result<RsvgNode, RenderingError> {
+    fn get_node_or_root(&self, id: Option<&str>) -> Result<Node, RenderingError> {
         if let Some(id) = id {
             self.lookup_node(id).map_err(RenderingError::InvalidId)
         } else {
@@ -187,7 +189,7 @@ impl Handle {
         ))
     }
 
-    fn lookup_node(&self, id: &str) -> Result<RsvgNode, DefsLookupErrorKind> {
+    fn lookup_node(&self, id: &str) -> Result<Node, DefsLookupErrorKind> {
         match Href::parse(&id).map_err(DefsLookupErrorKind::HrefError)? {
             Href::PlainUrl(_) => Err(DefsLookupErrorKind::CannotLookupExternalReferences),
             Href::WithFragment(fragment) => {
@@ -274,7 +276,7 @@ impl Handle {
 
     fn get_bbox_for_element(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         dpi: Dpi,
         is_testing: bool,
     ) -> Result<BoundingBox, RenderingError> {
@@ -366,7 +368,7 @@ impl Handle {
     }
 
     pub fn get_intrinsic_dimensions(&self) -> IntrinsicDimensions {
-        self.document.get_intrinsic_dimensions()
+        borrow_element_as!(self.document.root(), Svg).get_intrinsic_dimensions()
     }
 
     pub fn set_stylesheet(&mut self, css: &str) -> Result<(), LoadingError> {
@@ -386,27 +388,6 @@ fn check_cairo_context(cr: &cairo::Context) -> Result<(), RenderingError> {
     }
 }
 
-/// Gets the user's preferred locale from the environment and
-/// translates it to a `Locale` with `LanguageRange` fallbacks.
-///
-/// The `Locale::current()` call only contemplates a single language,
-/// but glib is smarter, and `g_get_langauge_names()` can provide
-/// fallbacks, for example, when LC_MESSAGES="en_US.UTF-8:de" (USA
-/// English and German).  This function converts the output of
-/// `g_get_language_names()` into a `Locale` with appropriate
-/// fallbacks.
-fn locale_from_environment() -> Locale {
-    let mut locale = Locale::invariant();
-
-    for name in glib::get_language_names() {
-        if let Ok(range) = LanguageRange::from_unix(&name) {
-            locale.add(&range);
-        }
-    }
-
-    locale
-}
-
 fn unit_rectangle() -> Rect {
     Rect::from_size(1.0, 1.0)
 }
