@@ -15,14 +15,17 @@
 //! to the drawing process.
 //!
 //! * [The `Document` struct](document/struct.Document.html) represents a loaded SVG
-//! document.  It holds the tree of [`RsvgNode`] elements, and a mapping of `id` attributes
+//! document.  It holds the tree of [`Node`] elements, and a mapping of `id` attributes
 //! to the corresponding element nodes.
 //!
-//! * [The `node` module](node/index.html) provides the [`RsvgNode`] struct and
-//! [`NodeTrait`], which form the basis for the tree of SVG elements.
+//! * [The `node` module](node/index.html) provides the [`Node`] struct and
+//! helper traits used to operate on nodes.
+//!
+//! * [The `element` module](element/index.html) provides the [`Element`] struct and
+//! the [`SetAttributes`] and [`Draw`] traits which are implemented by all SVG elements.
 //!
 //! * [The `xml` module](xml/index.html) receives events from the XML parser, and builds a
-//! [`Document`] as a tree of [`RsvgNode`].
+//! [`Document`] as a tree of [`Node`].
 //!
 //! * [The `properties` module](properties/index.html) contains structs that represent
 //! collections of CSS properties.
@@ -30,9 +33,14 @@
 //! * [The `property_defs` module](property_defs/index.html) contains one type for each of
 //! the CSS style properties that librsvg supports.
 //!
+//! * [The `css` module](css/index.html) contains the implementation of CSS parsing
+//! and matching.
+//!
 //! [`Document`]: document/struct.Document.html
-//! [`RsvgNode`]: node/type.RsvgNode.html
-//! [`NodeTrait`]: node/trait.NodeTrait.html
+//! [`Node`]: node/type.Node.html
+//! [`Element`]: element/struct.Element.html
+//! [`SetAttributes`]: element/trait.SetAttributes.html
+//! [`Draw`]: element/trait.Draw.html
 
 #![allow(clippy::clone_on_ref_ptr)]
 #![allow(clippy::not_unsafe_ptr_arg_deref)]
@@ -41,7 +49,7 @@
 
 pub use crate::color::Color;
 
-pub use crate::dpi::{rsvg_rust_set_default_dpi_x_y, Dpi};
+pub use crate::dpi::Dpi;
 
 pub use crate::error::{DefsLookupErrorKind, HrefError, LoadingError, RenderingError};
 
@@ -75,6 +83,9 @@ mod coord_units;
 #[macro_use]
 mod float_eq_cairo;
 
+#[macro_use]
+mod node;
+
 #[macro_use]
 mod property_macros;
 
@@ -84,12 +95,12 @@ mod aspect_ratio;
 mod bbox;
 mod color;
 mod cond;
-mod create_node;
 mod css;
 mod dasharray;
 mod document;
 mod dpi;
 mod drawing_ctx;
+mod element;
 mod error;
 mod filter;
 pub mod filters;
@@ -102,7 +113,6 @@ mod iri;
 mod length;
 mod limits;
 mod marker;
-mod node;
 mod number_list;
 mod paint_server;
 mod path_builder;
