@@ -12,14 +12,15 @@ use crate::aspect_ratio::*;
 use crate::bbox::BoundingBox;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
+use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::float_eq_cairo::ApproxEqCairo;
 use crate::iri::IRI;
 use crate::length::*;
-use crate::node::*;
+use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw};
 use crate::parsers::{Parse, ParseValue};
-use crate::path_builder::*;
-use crate::properties::{ComputedValues, SpecifiedValue, SpecifiedValues};
+use crate::path_builder::{arc_segment, ArcParameterization, CubicBezierCurve, Path, PathCommand};
+use crate::properties::ComputedValues;
 use crate::property_bag::PropertyBag;
 use crate::rect::Rect;
 use crate::transform::Transform;
@@ -99,7 +100,7 @@ impl Default for Marker {
 impl Marker {
     fn render(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         draw_ctx: &mut DrawingCtx,
         xpos: f64,
@@ -174,8 +175,8 @@ impl Marker {
     }
 }
 
-impl NodeTrait for Marker {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Marker {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "markerUnits") => self.units = attr.parse(value)?,
@@ -198,17 +199,10 @@ impl NodeTrait for Marker {
 
         Ok(())
     }
-
-    fn overflow_hidden(&self) -> bool {
-        true
-    }
-
-    fn set_overridden_properties(&self, values: &mut SpecifiedValues) {
-        // markers are always displayed, even if <marker> or its ancestors are display:none
-        values.display = SpecifiedValue::Specified(Default::default());
-    }
 }
 
+impl Draw for Marker {}
+
 // Machinery to figure out marker orientations
 #[derive(Debug, PartialEq)]
 enum Segment {
@@ -338,8 +332,8 @@ impl Deref for Segments {
 // The tangent at the end point is given by the vector (P4 - P3).
 // The tangents also work if the segment refers to a lineto (they will
 // both just point in the same direction).
-impl<'a> From<&'a PathBuilder> for Segments {
-    fn from(builder: &PathBuilder) -> Segments {
+impl From<&Path> for Segments {
+    fn from(path: &Path) -> Segments {
         let mut last_x: f64;
         let mut last_y: f64;
         let mut cur_x: f64;
@@ -357,11 +351,11 @@ impl<'a> From<&'a PathBuilder> for Segments {
         segments = Vec::new();
         state = SegmentState::Initial;
 
-        for path_command in builder.get_path_commands() {
+        for path_command in path.iter() {
             last_x = cur_x;
             last_y = cur_y;
 
-            match *path_command {
+            match path_command {
                 PathCommand::MoveTo(x, y) => {
                     cur_x = x;
                     cur_y = y;
@@ -570,19 +564,25 @@ fn emit_marker_by_name(
     line_width: f64,
     clipping: bool,
 ) -> Result<BoundingBox, RenderingError> {
-    if let Ok(acquired) = acquired_nodes.acquire(name, &[NodeType::Marker]) {
+    if let Ok(acquired) = acquired_nodes.acquire(name) {
         let node = acquired.get();
 
-        node.borrow().get_impl::<Marker>().render(
-            &node,
-            acquired_nodes,
-            draw_ctx,
-            xpos,
-            ypos,
-            computed_angle,
-            line_width,
-            clipping,
-        )
+        match *node.borrow_element() {
+            Element::Marker(ref m) => m.render(
+                &node,
+                acquired_nodes,
+                draw_ctx,
+                xpos,
+                ypos,
+                computed_angle,
+                line_width,
+                clipping,
+            ),
+            _ => {
+                rsvg_log!("\"{}\" is not marker", name);
+                Ok(draw_ctx.empty_bbox())
+            }
+        }
     } else {
         rsvg_log!("marker \"{}\" not found", name);
         Ok(draw_ctx.empty_bbox())
@@ -617,15 +617,15 @@ where
     emit_fn(marker_type, x, y, orient)
 }
 
-pub fn render_markers_for_path_builder(
-    builder: &PathBuilder,
+pub fn render_markers_for_path(
+    path: &Path,
     draw_ctx: &mut DrawingCtx,
     acquired_nodes: &mut AcquiredNodes,
     values: &ComputedValues,
     clipping: bool,
 ) -> Result<BoundingBox, RenderingError> {
     let line_width = values
-        .stroke_width
+        .stroke_width()
         .0
         .normalize(values, &draw_ctx.get_view_params());
 
@@ -633,22 +633,22 @@ pub fn render_markers_for_path_builder(
         return Ok(draw_ctx.empty_bbox());
     }
 
-    let marker_start = &values.marker_start.0;
-    let marker_mid = &values.marker_mid.0;
-    let marker_end = &values.marker_end.0;
+    let marker_start = values.marker_start().0;
+    let marker_mid = values.marker_mid().0;
+    let marker_end = values.marker_end().0;
 
-    if let (&IRI::None, &IRI::None, &IRI::None) = (marker_start, marker_mid, marker_end) {
+    if let (&IRI::None, &IRI::None, &IRI::None) = (&marker_start, &marker_mid, &marker_end) {
         return Ok(draw_ctx.empty_bbox());
     }
 
-    emit_markers_for_path_builder(
-        builder,
+    emit_markers_for_path(
+        path,
         draw_ctx.empty_bbox(),
         &mut |marker_type: MarkerType, x: f64, y: f64, computed_angle: Angle| {
-            if let &IRI::Resource(ref marker) = match marker_type {
-                MarkerType::Start => &values.marker_start.0,
-                MarkerType::Middle => &values.marker_mid.0,
-                MarkerType::End => &values.marker_end.0,
+            if let IRI::Resource(ref marker) = match marker_type {
+                MarkerType::Start => &marker_start,
+                MarkerType::Middle => &marker_mid,
+                MarkerType::End => &marker_end,
             } {
                 emit_marker_by_name(
                     draw_ctx,
@@ -667,8 +667,8 @@ pub fn render_markers_for_path_builder(
     )
 }
 
-fn emit_markers_for_path_builder<E>(
-    builder: &PathBuilder,
+fn emit_markers_for_path<E>(
+    path: &Path,
     empty_bbox: BoundingBox,
     emit_fn: &mut E,
 ) -> Result<BoundingBox, RenderingError>
@@ -683,7 +683,7 @@ where
     let mut bbox = empty_bbox;
 
     // Convert the path to a list of segments and bare points
-    let segments = Segments::from(builder);
+    let segments = Segments::from(path);
 
     let mut subpath_state = SubpathState::NoSubpath;
 
@@ -775,7 +775,7 @@ where
                 .unwrap_or_else(|| Angle::new(0.0));
 
             let angle = {
-                if let PathCommand::ClosePath = builder.get_path_commands()[segments.len()] {
+                if let PathCommand::ClosePath = path.iter().nth(segments.len()).unwrap() {
                     let outgoing = segments
                         .find_outgoing_angle_forwards(0)
                         .unwrap_or_else(|| Angle::new(0.0));
@@ -858,6 +858,7 @@ mod parser_tests {
 #[cfg(test)]
 mod directionality_tests {
     use super::*;
+    use crate::path_builder::PathBuilder;
 
     // Single open path; the easy case
     fn setup_open_path() -> Segments {
@@ -867,7 +868,7 @@ mod directionality_tests {
         builder.line_to(20.0, 10.0);
         builder.line_to(20.0, 20.0);
 
-        Segments::from(&builder)
+        Segments::from(&builder.into_path())
     }
 
     #[test]
@@ -892,7 +893,7 @@ mod directionality_tests {
         builder.curve_to(50.0, 35.0, 60.0, 60.0, 70.0, 70.0);
         builder.line_to(80.0, 90.0);
 
-        Segments::from(&builder)
+        Segments::from(&builder.into_path())
     }
 
     #[test]
@@ -917,7 +918,7 @@ mod directionality_tests {
         builder.line_to(20.0, 20.0);
         builder.close_path();
 
-        Segments::from(&builder)
+        Segments::from(&builder.into_path())
     }
 
     #[test]
@@ -947,7 +948,7 @@ mod directionality_tests {
         builder.line_to(80.0, 90.0);
         builder.close_path();
 
-        Segments::from(&builder)
+        Segments::from(&builder.into_path())
     }
 
     #[test]
@@ -977,7 +978,7 @@ mod directionality_tests {
 
         builder.line_to(40.0, 30.0);
 
-        Segments::from(&builder)
+        Segments::from(&builder.into_path())
     }
 
     #[test]
@@ -1011,7 +1012,7 @@ mod directionality_tests {
     // builder.move_to (30.0, 30.0);
     // builder.move_to (40.0, 40.0);
     //
-    // Segments::from(&builder)
+    // Segments::from(&builder.into_path())
     // }
     //
     // #[test]
@@ -1106,6 +1107,7 @@ mod directionality_tests {
 #[cfg(test)]
 mod marker_tests {
     use super::*;
+    use crate::path_builder::PathBuilder;
 
     #[test]
     fn emits_for_open_subpath() {
@@ -1117,8 +1119,8 @@ mod marker_tests {
 
         let mut v = Vec::new();
 
-        assert!(emit_markers_for_path_builder(
-            &builder,
+        assert!(emit_markers_for_path(
+            &builder.into_path(),
             BoundingBox::new(),
             &mut |marker_type: MarkerType,
                   x: f64,
@@ -1153,8 +1155,8 @@ mod marker_tests {
 
         let mut v = Vec::new();
 
-        assert!(emit_markers_for_path_builder(
-            &builder,
+        assert!(emit_markers_for_path(
+            &builder.into_path(),
             BoundingBox::new(),
             &mut |marker_type: MarkerType,
                   x: f64,
