@@ -1,7 +1,10 @@
 //! Representation of Bézier paths.
 
+use tinyvec::TinyVec;
+
 use std::f64;
 use std::f64::consts::*;
+use std::slice;
 
 use crate::float_eq_cairo::ApproxEqCairo;
 use crate::util::clamp;
@@ -15,7 +18,7 @@ pub enum Sweep {
     Positive,
 }
 
-#[derive(Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq, Default)]
 pub struct CubicBezierCurve {
     /// The (x, y) coordinates of the first control point.
     pub pt1: (f64, f64),
@@ -30,10 +33,28 @@ impl CubicBezierCurve {
         let Self { pt1, pt2, to } = self;
         cr.curve_to(pt1.0, pt1.1, pt2.0, pt2.1, to.0, to.1);
     }
+
+    fn from_coords<'a>(coords: &mut slice::Iter<'a, f64>) -> CubicBezierCurve {
+        let pt1 = take_two(coords);
+        let pt2 = take_two(coords);
+        let to = take_two(coords);
+
+        CubicBezierCurve { pt1, pt2, to }
+    }
+
+    fn to_packed_and_coords(&self, coords: &mut Vec<f64>) -> PackedCommand {
+        coords.push(self.pt1.0);
+        coords.push(self.pt1.1);
+        coords.push(self.pt2.0);
+        coords.push(self.pt2.1);
+        coords.push(self.to.0);
+        coords.push(self.to.1);
+        PackedCommand::CurveTo
+    }
 }
 
 /// When attempting to compute the center parameterization of the arc,
-/// out of range parameters may see an arc omitted or treated as a line.
+/// out of range parameters may see an arc omited or treated as a line.
 pub enum ArcParameterization {
     /// Center parameterization of the arc.
     CenterParameters {
@@ -223,6 +244,43 @@ impl EllipticalArc {
             ArcParameterization::Omit => {}
         }
     }
+
+    fn from_coords<'a>(
+        large_arc: LargeArc,
+        sweep: Sweep,
+        coords: &mut slice::Iter<'a, f64>,
+    ) -> EllipticalArc {
+        let r = take_two(coords);
+        let x_axis_rotation = take_one(coords);
+        let from = take_two(coords);
+        let to = take_two(coords);
+
+        EllipticalArc {
+            r,
+            x_axis_rotation,
+            large_arc,
+            sweep,
+            from,
+            to,
+        }
+    }
+
+    fn to_packed_and_coords(&self, coords: &mut Vec<f64>) -> PackedCommand {
+        coords.push(self.r.0);
+        coords.push(self.r.1);
+        coords.push(self.x_axis_rotation);
+        coords.push(self.from.0);
+        coords.push(self.from.1);
+        coords.push(self.to.0);
+        coords.push(self.to.1);
+
+        match (self.large_arc, self.sweep) {
+            (LargeArc(false), Sweep::Negative) => PackedCommand::ArcSmallNegative,
+            (LargeArc(false), Sweep::Positive) => PackedCommand::ArcSmallPositive,
+            (LargeArc(true), Sweep::Negative) => PackedCommand::ArcLargeNegative,
+            (LargeArc(true), Sweep::Positive) => PackedCommand::ArcLargePositive,
+        }
+    }
 }
 
 /// Turns an arc segment into a cubic bezier curve.
@@ -278,6 +336,12 @@ pub enum PathCommand {
     ClosePath,
 }
 
+impl Default for PathCommand {
+    fn default() -> Self {
+        PathCommand::CurveTo(CubicBezierCurve::default())
+    }
+}
+
 impl PathCommand {
     fn to_cairo(&self, cr: &cairo::Context) {
         match *self {
@@ -288,16 +352,151 @@ impl PathCommand {
             PathCommand::ClosePath => cr.close_path(),
         }
     }
+
+    // Returns the number of coordinate values that this command will generate in a `Path`.
+    fn num_coordinates(&self) -> usize {
+        match *self {
+            PathCommand::MoveTo(..) => 2,
+            PathCommand::LineTo(..) => 2,
+            PathCommand::CurveTo(_) => 6,
+            PathCommand::Arc(_) => 7,
+            PathCommand::ClosePath => 0,
+        }
+    }
+
+    fn to_packed(&self, coords: &mut Vec<f64>) -> PackedCommand {
+        match *self {
+            PathCommand::MoveTo(x, y) => {
+                coords.push(x);
+                coords.push(y);
+                PackedCommand::MoveTo
+            }
+
+            PathCommand::LineTo(x, y) => {
+                coords.push(x);
+                coords.push(y);
+                PackedCommand::LineTo
+            }
+
+            PathCommand::CurveTo(ref c) => c.to_packed_and_coords(coords),
+
+            PathCommand::Arc(ref a) => a.to_packed_and_coords(coords),
+
+            PathCommand::ClosePath => PackedCommand::ClosePath,
+        }
+    }
+
+    fn from_packed<'a>(packed: PackedCommand, coords: &mut slice::Iter<'a, f64>) -> PathCommand {
+        match packed {
+            PackedCommand::MoveTo => {
+                let x = take_one(coords);
+                let y = take_one(coords);
+                PathCommand::MoveTo(x, y)
+            }
+
+            PackedCommand::LineTo => {
+                let x = take_one(coords);
+                let y = take_one(coords);
+                PathCommand::LineTo(x, y)
+            }
+
+            PackedCommand::CurveTo => PathCommand::CurveTo(CubicBezierCurve::from_coords(coords)),
+
+            PackedCommand::ClosePath => PathCommand::ClosePath,
+
+            PackedCommand::ArcSmallNegative => PathCommand::Arc(EllipticalArc::from_coords(
+                LargeArc(false),
+                Sweep::Negative,
+                coords,
+            )),
+
+            PackedCommand::ArcSmallPositive => PathCommand::Arc(EllipticalArc::from_coords(
+                LargeArc(false),
+                Sweep::Positive,
+                coords,
+            )),
+
+            PackedCommand::ArcLargeNegative => PathCommand::Arc(EllipticalArc::from_coords(
+                LargeArc(true),
+                Sweep::Negative,
+                coords,
+            )),
+
+            PackedCommand::ArcLargePositive => PathCommand::Arc(EllipticalArc::from_coords(
+                LargeArc(true),
+                Sweep::Positive,
+                coords,
+            )),
+        }
+    }
 }
 
-#[derive(Clone, Default)]
+/// Constructs a path out of commands.
+///
+/// When you are finished constructing a path builder, turn it into
+/// a `Path` with `into_path`.
+#[derive(Clone)]
 pub struct PathBuilder {
-    path_commands: Vec<PathCommand>,
+    path_commands: TinyVec<[PathCommand; 32]>,
+}
+
+/// An immutable path with a compact representation.
+///
+/// This is constructed from a `PathBuilder` once it is finished.  You
+/// can get an iterator for the path's commands with the `iter`
+/// function.
+///
+/// Most `PathCommand` variants only have a few coordinates, but `PathCommand::Arc`
+/// has two extra booleans.  We separate the commands from their coordinates so
+/// we can have two dense arrays: one with a compact representation of commands,
+/// and another with a linear list of the coordinates for each command.
+///
+/// Each `PathCommand` knows how many coordinates it ought to produce, with
+/// its `num_coordinates` method.
+pub struct Path {
+    commands: Box<[PackedCommand]>,
+    coords: Box<[f64]>,
+}
+
+/// Packed version of a `PathCommand`, used in `Path`.
+#[repr(u8)]
+#[derive(Debug, Clone, Copy)]
+enum PackedCommand {
+    MoveTo,
+    LineTo,
+    CurveTo,
+    ArcSmallNegative,
+    ArcSmallPositive,
+    ArcLargeNegative,
+    ArcLargePositive,
+    ClosePath,
 }
 
 impl PathBuilder {
     pub fn new() -> PathBuilder {
-        PathBuilder::default()
+        PathBuilder {
+            path_commands: TinyVec::new(),
+        }
+    }
+
+    pub fn into_path(self) -> Path {
+        let num_coords = self
+            .path_commands
+            .iter()
+            .map(PathCommand::num_coordinates)
+            .sum();
+
+        let mut coords = Vec::with_capacity(num_coords);
+        let packed_commands: Vec<_> = self
+            .path_commands
+            .iter()
+            .map(|cmd| cmd.to_packed(&mut coords))
+            .collect();
+
+        Path {
+            commands: packed_commands.into_boxed_slice(),
+            coords: coords.into_boxed_slice(),
+        }
     }
 
     pub fn move_to(&mut self, x: f64, y: f64) {
@@ -343,19 +542,24 @@ impl PathBuilder {
     pub fn close_path(&mut self) {
         self.path_commands.push(PathCommand::ClosePath);
     }
+}
 
-    pub fn get_path_commands(&self) -> &[PathCommand] {
-        &self.path_commands
+impl Path {
+    pub fn iter<'a>(&'a self) -> impl Iterator<Item = PathCommand> + 'a {
+        let commands = self.commands.iter();
+        let mut coords = self.coords.iter();
+
+        commands.map(move |cmd| PathCommand::from_packed(*cmd, &mut coords))
     }
 
     pub fn is_empty(&self) -> bool {
-        self.path_commands.is_empty()
+        self.commands.is_empty()
     }
 
     pub fn to_cairo(&self, cr: &cairo::Context) -> Result<(), cairo::Status> {
         assert!(!self.is_empty());
 
-        for s in &self.path_commands {
+        for s in self.iter() {
             s.to_cairo(cr);
         }
 
@@ -377,25 +581,62 @@ impl PathBuilder {
     }
 }
 
+fn take_one<'a>(iter: &mut slice::Iter<'a, f64>) -> f64 {
+    *iter.next().unwrap()
+}
+
+fn take_two<'a>(iter: &mut slice::Iter<'a, f64>) -> (f64, f64) {
+    (take_one(iter), take_one(iter))
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
 
     #[test]
-    fn survives_degenerate_arcs() {
+    fn empty_builder() {
+        let builder = PathBuilder::new();
+        let path = builder.into_path();
+        assert!(path.is_empty());
+        assert_eq!(path.iter().count(), 0);
+    }
+
+    #[test]
+    fn all_commands() {
         let mut builder = PathBuilder::new();
-        builder.move_to(0.0, 0.0);
-        // Deliberately close to 0 to try to trigger division by 0.
+        builder.move_to(42.0, 43.0);
+        builder.line_to(42.0, 43.0);
+        builder.curve_to(42.0, 43.0, 44.0, 45.0, 46.0, 47.0);
         builder.arc(
-            0.0,
-            0.0,
-            f64::EPSILON,
-            f64::EPSILON,
-            0.0,
+            42.0,
+            43.0,
+            44.0,
+            45.0,
+            46.0,
             LargeArc(true),
             Sweep::Positive,
-            1.0,
-            1.0,
+            47.0,
+            48.0,
         );
+        builder.close_path();
+        let path = builder.into_path();
+        assert!(path.iter().eq(vec![
+            PathCommand::MoveTo(42.0, 43.0),
+            PathCommand::LineTo(42.0, 43.0),
+            PathCommand::CurveTo(CubicBezierCurve {
+                pt1: (42.0, 43.0),
+                pt2: (44.0, 45.0),
+                to: (46.0, 47.0),
+            }),
+            PathCommand::Arc(EllipticalArc {
+                from: (42.0, 43.0),
+                r: (44.0, 45.0),
+                to: (47.0, 48.0),
+                x_axis_rotation: 46.0,
+                large_arc: LargeArc(true),
+                sweep: Sweep::Positive,
+            }),
+            PathCommand::ClosePath,
+        ]));
     }
 }
