@@ -39,7 +39,7 @@ struct PathParser<'b> {
 // as specified in https://www.w3.org/TR/SVG/paths.html#PathDataBNF
 // Some peculiarities:
 //
-// - SVG allows optional commas inside coordinate pairs, and between
+// - SVG allows optional commas inside coordiante pairs, and between
 // coordinate pairs.  So, for example, these are equivalent:
 //
 //     M 10 20 30 40
@@ -968,9 +968,10 @@ mod tests {
         let mut builder = PathBuilder::new();
         let result = parse_path_into_builder(path_str, &mut builder);
 
-        let commands = builder.get_path_commands();
+        let path = builder.into_path();
+        let commands = path.iter().collect::<Vec<_>>();
 
-        assert_eq!(expected_commands, commands);
+        assert_eq!(expected_commands, commands.as_slice());
         assert_eq!(expected_result, result);
     }
 
