@@ -10,10 +10,11 @@ use crate::bbox::*;
 use crate::coord_units::CoordUnits;
 use crate::document::{AcquiredNodes, NodeStack};
 use crate::drawing_ctx::{DrawingCtx, ViewParams};
+use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::float_eq_cairo::ApproxEqCairo;
 use crate::length::*;
-use crate::node::*;
+use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw, WeakNode};
 use crate::paint_server::{AsPaintSource, PaintSource};
 use crate::parsers::ParseValue;
 use crate::properties::ComputedValues;
@@ -65,7 +66,7 @@ enum UnresolvedChildren {
     ResolvedEmpty,
 
     /// Points back to the Pattern that had usable children.
-    WithChildren(RsvgWeakNode),
+    WithChildren(WeakNode),
 }
 
 /// Keeps track of which Pattern provided a non-empty set of children during pattern resolution
@@ -74,7 +75,7 @@ enum Children {
     Empty,
 
     /// Points back to the Pattern that had usable children
-    WithChildren(RsvgWeakNode),
+    WithChildren(WeakNode),
 }
 
 /// Main structure used during pattern resolution.  For unresolved
@@ -117,8 +118,8 @@ pub struct Pattern {
     resolved: RefCell<Option<ResolvedPattern>>,
 }
 
-impl NodeTrait for Pattern {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Pattern {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "patternUnits") => self.common.units = Some(attr.parse(value)?),
@@ -152,18 +153,16 @@ impl NodeTrait for Pattern {
 
         Ok(())
     }
-
-    fn overflow_hidden(&self) -> bool {
-        true
-    }
 }
 
+impl Draw for Pattern {}
+
 impl PaintSource for Pattern {
     type Resolved = ResolvedPattern;
 
     fn resolve(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
     ) -> Result<Self::Resolved, AcquireError> {
         let mut resolved = self.resolved.borrow_mut();
@@ -180,7 +179,7 @@ impl PaintSource for Pattern {
 
         while !pattern.is_resolved() {
             if let Some(ref fragment) = fallback {
-                match acquired_nodes.acquire(&fragment, &[NodeType::Pattern]) {
+                match acquired_nodes.acquire(&fragment) {
                     Ok(acquired) => {
                         let acquired_node = acquired.get();
 
@@ -188,14 +187,16 @@ impl PaintSource for Pattern {
                             return Err(AcquireError::CircularReference(acquired_node.clone()));
                         }
 
-                        let borrowed_node = acquired_node.borrow();
-                        let borrowed_pattern = borrowed_node.get_impl::<Pattern>();
-                        let unresolved = borrowed_pattern.get_unresolved(&acquired_node);
+                        match *acquired_node.borrow_element() {
+                            Element::Pattern(ref p) => {
+                                let unresolved = p.get_unresolved(&acquired_node);
+                                pattern = pattern.resolve_from_fallback(&unresolved.pattern);
+                                fallback = unresolved.fallback;
 
-                        pattern = pattern.resolve_from_fallback(&unresolved.pattern);
-                        fallback = unresolved.fallback;
-
-                        stack.push(acquired_node);
+                                stack.push(acquired_node);
+                            }
+                            _ => return Err(AcquireError::InvalidLinkType(fragment.clone())),
+                        }
                     }
 
                     Err(AcquireError::MaxReferencesExceeded) => {
@@ -475,13 +476,10 @@ impl UnresolvedPattern {
 }
 
 impl UnresolvedChildren {
-    fn from_node(node: &RsvgNode) -> UnresolvedChildren {
+    fn from_node(node: &Node) -> UnresolvedChildren {
         let weak = node.downgrade();
 
-        if node
-            .children()
-            .any(|child| child.borrow().get_type() != NodeType::Chars)
-        {
+        if node.children().any(|child| child.is_element()) {
             UnresolvedChildren::WithChildren(weak)
         } else {
             UnresolvedChildren::Unresolved
@@ -529,7 +527,7 @@ impl UnresolvedChildren {
 }
 
 impl Children {
-    fn node_with_children(&self) -> Option<RsvgNode> {
+    fn node_with_children(&self) -> Option<Node> {
         match *self {
             Children::Empty => None,
             Children::WithChildren(ref wc) => Some(wc.upgrade().unwrap()),
@@ -549,7 +547,7 @@ impl ResolvedPattern {
 }
 
 impl Pattern {
-    fn get_unresolved(&self, node: &RsvgNode) -> Unresolved {
+    fn get_unresolved(&self, node: &Node) -> Unresolved {
         let pattern = UnresolvedPattern {
             common: self.common.clone(),
             children: UnresolvedChildren::from_node(node),
@@ -565,23 +563,21 @@ impl Pattern {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::node::{NodeData, NodeType, RsvgNode};
+    use crate::node::NodeData;
     use markup5ever::{namespace_url, ns, QualName};
+    use std::ptr;
 
     #[test]
     fn pattern_resolved_from_defaults_is_really_resolved() {
-        let node = RsvgNode::new(NodeData::new(
-            NodeType::Pattern,
+        let bag = unsafe { PropertyBag::new_from_xml2_attributes(0, ptr::null()) };
+
+        let node = Node::new(NodeData::new_element(
             &QualName::new(None, ns!(svg), local_name!("pattern")),
-            None,
-            None,
-            Box::new(Pattern::default()),
+            &bag,
         ));
 
-        let borrow = node.borrow();
-        let p = borrow.get_impl::<Pattern>();
-        let Unresolved { pattern, .. } = p.get_unresolved(&node);
-        let pattern = pattern.resolve_from_defaults();
+        let unresolved = borrow_element_as!(node, Pattern).get_unresolved(&node);
+        let pattern = unresolved.pattern.resolve_from_defaults();
         assert!(pattern.is_resolved());
     }
 }
