@@ -8,11 +8,12 @@ use crate::allowed_url::Fragment;
 use crate::bbox::BoundingBox;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
+use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::float_eq_cairo::ApproxEqCairo;
 use crate::font_props::FontWeightSpec;
 use crate::length::*;
-use crate::node::{CascadedValues, NodeResult, NodeTrait, NodeType, RsvgNode};
+use crate::node::{CascadedValues, Node, NodeBorrow};
 use crate::parsers::ParseValue;
 use crate::properties::ComputedValues;
 use crate::property_bag::PropertyBag;
@@ -39,15 +40,15 @@ use crate::transform::Transform;
 /// [text chunk]: https://www.w3.org/TR/SVG11/text.html#TextLayoutIntroduction
 struct Chunk {
     values: ComputedValues,
-    x: Option<f64>,
-    y: Option<f64>,
+    x: Option<Length<Horizontal>>,
+    y: Option<Length<Vertical>>,
     spans: Vec<Span>,
 }
 
 struct MeasuredChunk {
     values: ComputedValues,
-    x: Option<f64>,
-    y: Option<f64>,
+    x: Option<Length<Horizontal>>,
+    y: Option<Length<Vertical>>,
     advance: (f64, f64),
     spans: Vec<MeasuredSpan>,
 }
@@ -61,8 +62,8 @@ struct PositionedChunk {
 struct Span {
     values: ComputedValues,
     text: String,
-    dx: f64,
-    dy: f64,
+    dx: Option<Length<Horizontal>>,
+    dy: Option<Length<Vertical>>,
     _depth: usize,
 }
 
@@ -71,8 +72,8 @@ struct MeasuredSpan {
     layout: pango::Layout,
     _layout_size: (f64, f64),
     advance: (f64, f64),
-    dx: f64,
-    dy: f64,
+    dx: Option<Length<Horizontal>>,
+    dy: Option<Length<Vertical>>,
 }
 
 struct PositionedSpan {
@@ -85,7 +86,11 @@ struct PositionedSpan {
 }
 
 impl Chunk {
-    fn new(values: &ComputedValues, x: Option<f64>, y: Option<f64>) -> Chunk {
+    fn new(
+        values: &ComputedValues,
+        x: Option<Length<Horizontal>>,
+        y: Option<Length<Vertical>>,
+    ) -> Chunk {
         Chunk {
             values: values.clone(),
             x,
@@ -129,8 +134,8 @@ impl PositionedChunk {
         // Adjust the specified coordinates with the text_anchor
 
         let adjusted_advance = text_anchor_advance(
-            measured.values.text_anchor,
-            measured.values.writing_mode,
+            measured.values.text_anchor(),
+            measured.values.writing_mode(),
             measured.advance,
         );
 
@@ -177,7 +182,13 @@ fn text_anchor_advance(
 }
 
 impl Span {
-    fn new(text: &str, values: ComputedValues, dx: f64, dy: f64, depth: usize) -> Span {
+    fn new(
+        text: &str,
+        values: ComputedValues,
+        dx: Option<Length<Horizontal>>,
+        dy: Option<Length<Vertical>>,
+        depth: usize,
+    ) -> Span {
         Span {
             values,
             text: text.to_string(),
@@ -198,7 +209,7 @@ impl MeasuredSpan {
         let w = f64::from(w) / f64::from(pango::SCALE);
         let h = f64::from(h) / f64::from(pango::SCALE);
 
-        let advance = if values.writing_mode.is_vertical() {
+        let advance = if values.writing_mode().is_vertical() {
             (0.0, w)
         } else {
             (w, 0.0)
@@ -228,13 +239,19 @@ impl PositionedSpan {
         let params = draw_ctx.get_view_params();
 
         let baseline = f64::from(layout.get_baseline()) / f64::from(pango::SCALE);
-        let baseline_shift = values.baseline_shift.0.normalize(&values, &params);
+        let baseline_shift = values.baseline_shift().0.normalize(&values, &params);
         let offset = baseline + baseline_shift;
 
-        let dx = measured.dx;
-        let dy = measured.dy;
+        let dx = measured
+            .dx
+            .map(|l| l.normalize(&values, &params))
+            .unwrap_or(0.0);
+        let dy = measured
+            .dy
+            .map(|l| l.normalize(&values, &params))
+            .unwrap_or(0.0);
 
-        let (render_x, render_y) = if values.writing_mode.is_vertical() {
+        let (render_x, render_y) = if values.writing_mode().is_vertical() {
             (x + offset + dx, y + dy)
         } else {
             (x + dx, y - offset + dy)
@@ -272,7 +289,7 @@ impl PositionedSpan {
             };
 
             let cr = dc.get_cairo_context();
-            cr.set_antialias(cairo::Antialias::from(self.values.text_rendering));
+            cr.set_antialias(cairo::Antialias::from(self.values.text_rendering()));
             dc.setup_cr_for_stroke(&cr, &self.values);
             cr.move_to(self.rendered_position.0, self.rendered_position.1);
 
@@ -281,13 +298,13 @@ impl PositionedSpan {
                 cr.rotate(-rotation);
             }
 
-            let current_color = self.values.color.0;
+            let current_color = self.values.color().0;
 
             let res = if !clipping {
                 dc.set_source_paint_server(
                     acquired_nodes,
-                    &self.values.fill.0,
-                    self.values.fill_opacity.0,
+                    &self.values.fill().0,
+                    self.values.fill_opacity().0,
                     &bbox,
                     current_color,
                 )
@@ -308,8 +325,8 @@ impl PositionedSpan {
                 let res = if !clipping {
                     dc.set_source_paint_server(
                         acquired_nodes,
-                        &self.values.stroke.0,
-                        self.values.stroke_opacity.0,
+                        &self.values.stroke().0,
+                        self.values.stroke_opacity().0,
                         &bbox,
                         current_color,
                     )
@@ -403,85 +420,78 @@ fn gravity_is_vertical(gravity: pango::Gravity) -> bool {
 /// will be used instead of the given arguments.
 fn children_to_chunks(
     chunks: &mut Vec<Chunk>,
-    node: &RsvgNode,
+    node: &Node,
     acquired_nodes: &mut AcquiredNodes,
     cascaded: &CascadedValues<'_>,
     draw_ctx: &mut DrawingCtx,
-    dx: f64,
-    dy: f64,
+    dx: Option<Length<Horizontal>>,
+    dy: Option<Length<Vertical>>,
     depth: usize,
 ) {
     for child in node.children() {
-        match child.borrow().get_type() {
-            NodeType::Chars => {
-                let values = cascaded.get();
-                child
-                    .borrow()
-                    .get_impl::<NodeChars>()
-                    .to_chunks(&child, values, chunks, dx, dy, depth);
-            }
+        if child.is_chars() {
+            let values = cascaded.get();
+            child
+                .borrow_chars()
+                .to_chunks(&child, values, chunks, dx, dy, depth);
+        } else {
+            assert!(child.is_element());
 
-            NodeType::TSpan => {
-                let cascaded = CascadedValues::new(cascaded, &child);
-                child.borrow().get_impl::<TSpan>().to_chunks(
-                    &child,
-                    acquired_nodes,
-                    &cascaded,
-                    draw_ctx,
-                    chunks,
-                    dx,
-                    dy,
-                    depth + 1,
-                );
-            }
+            match *child.borrow_element() {
+                Element::TSpan(ref tspan) => {
+                    let cascaded = CascadedValues::new(cascaded, &child);
+                    tspan.to_chunks(
+                        &child,
+                        acquired_nodes,
+                        &cascaded,
+                        draw_ctx,
+                        chunks,
+                        depth + 1,
+                    );
+                }
 
-            NodeType::TRef => {
-                let cascaded = CascadedValues::new(cascaded, &child);
-                child.borrow().get_impl::<TRef>().to_chunks(
-                    &child,
-                    acquired_nodes,
-                    &cascaded,
-                    chunks,
-                    depth + 1,
-                );
-            }
+                Element::TRef(ref tref) => {
+                    let cascaded = CascadedValues::new(cascaded, &child);
+                    tref.to_chunks(&child, acquired_nodes, &cascaded, chunks, depth + 1);
+                }
 
-            _ => (),
+                _ => (),
+            }
         }
     }
 }
 
-/// In SVG text elements, we use `NodeChars` to store character data.  For example,
+/// In SVG text elements, we use `Chars` to store character data.  For example,
 /// an element like `<text>Foo Bar</text>` will be a `Text` with a single child,
-/// and the child will be a `NodeChars` with "Foo Bar" for its contents.
+/// and the child will be a `Chars` with "Foo Bar" for its contents.
 ///
 /// Text elements can contain `<tspan>` sub-elements.  In this case,
-/// those `tspan` nodes will also contain `NodeChars` children.
+/// those `tspan` nodes will also contain `Chars` children.
 ///
-/// A text or tspan element can contain more than one `NodeChars` child, for example,
+/// A text or tspan element can contain more than one `Chars` child, for example,
 /// if there is an XML comment that splits the character contents in two:
 ///
 /// ```xml
 /// <text>
-///   This sentence will create a NodeChars.
+///   This sentence will create a Chars.
 ///   <!-- this comment is ignored -->
-///   This sentence will cretea another NodeChars.
+///   This sentence will cretea another Chars.
 /// </text>
 /// ```
 ///
 /// When rendering a text element, it will take care of concatenating the strings
-/// in its `NodeChars` children as appropriate, depending on the
-/// `xml:space="preserve"` attribute.  A `NodeChars` stores the characters verbatim
+/// in its `Chars` children as appropriate, depending on the
+/// `xml:space="preserve"` attribute.  A `Chars` stores the characters verbatim
 /// as they come out of the XML parser, after ensuring that they are valid UTF-8.
 
-pub struct NodeChars {
+pub struct Chars {
     string: RefCell<String>,
     space_normalized: RefCell<Option<String>>,
 }
 
-impl NodeChars {
-    pub fn new() -> NodeChars {
-        NodeChars {
+impl Chars {
+    pub fn new() -> Chars {
+        Chars {
             string: RefCell::new(String::new()),
             space_normalized: RefCell::new(None),
         }
@@ -496,11 +506,11 @@ impl NodeChars {
         *self.space_normalized.borrow_mut() = None;
     }
 
-    fn ensure_normalized_string(&self, node: &RsvgNode, values: &ComputedValues) {
+    fn ensure_normalized_string(&self, node: &Node, values: &ComputedValues) {
         let mut normalized = self.space_normalized.borrow_mut();
 
         if (*normalized).is_none() {
-            let mode = match values.xml_space {
+            let mode = match values.xml_space() {
                 XmlSpace::Default => XmlSpaceNormalize::Default(NormalizeDefault {
                     has_element_before: node.previous_sibling().is_some(),
                     has_element_after: node.next_sibling().is_some(),
@@ -515,42 +525,38 @@ impl NodeChars {
 
     fn make_span(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         values: &ComputedValues,
-        dx: f64,
-        dy: f64,
+        dx: Option<Length<Horizontal>>,
+        dy: Option<Length<Vertical>>,
         depth: usize,
-    ) -> Option<Span> {
+    ) -> Span {
         self.ensure_normalized_string(node, values);
 
-        if self.space_normalized.borrow().as_ref().unwrap() == "" {
-            None
-        } else {
-            Some(Span::new(
-                self.space_normalized.borrow().as_ref().unwrap(),
-                values.clone(),
-                dx,
-                dy,
-                depth,
-            ))
-        }
+        Span::new(
+            self.space_normalized.borrow().as_ref().unwrap(),
+            values.clone(),
+            dx,
+            dy,
+            depth,
+        )
     }
 
     fn to_chunks(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         values: &ComputedValues,
         chunks: &mut Vec<Chunk>,
-        dx: f64,
-        dy: f64,
+        dx: Option<Length<Horizontal>>,
+        dy: Option<Length<Vertical>>,
         depth: usize,
     ) {
-        if let Some(span) = self.make_span(&node, values, dx, dy, depth) {
-            let num_chunks = chunks.len();
-            assert!(num_chunks > 0);
+        let span = self.make_span(&node, values, dx, dy, depth);
 
-            chunks[num_chunks - 1].spans.push(span);
-        }
+        let num_chunks = chunks.len();
+        assert!(num_chunks > 0);
+
+        chunks[num_chunks - 1].spans.push(span);
     }
 
     pub fn get_string(&self) -> String {
@@ -558,12 +564,6 @@ impl NodeChars {
     }
 }
 
-impl NodeTrait for NodeChars {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, _: &PropertyBag<'_>) -> NodeResult {
-        Ok(())
-    }
-}
-
 #[derive(Default)]
 pub struct Text {
     x: Length<Horizontal>,
@@ -575,39 +575,29 @@ pub struct Text {
 impl Text {
     fn make_chunks(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
-        x: f64,
-        y: f64,
     ) -> Vec<Chunk> {
         let mut chunks = Vec::new();
-
-        let values = cascaded.get();
-        let params = draw_ctx.get_view_params();
-
-        chunks.push(Chunk::new(&values, Some(x), Some(y)));
-
-        let dx = self.dx.map_or(0.0, |l| l.normalize(&values, &params));
-        let dy = self.dy.map_or(0.0, |l| l.normalize(&values, &params));
-
+        chunks.push(Chunk::new(cascaded.get(), Some(self.x), Some(self.y)));
         children_to_chunks(
             &mut chunks,
             node,
             acquired_nodes,
             cascaded,
             draw_ctx,
-            dx,
-            dy,
+            self.dx,
+            self.dy,
             0,
         );
         chunks
     }
 }
 
-impl NodeTrait for Text {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Text {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "x") => self.x = attr.parse(value)?,
@@ -620,10 +610,12 @@ impl NodeTrait for Text {
 
         Ok(())
     }
+}
 
+impl Draw for Text {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
@@ -635,7 +627,7 @@ impl NodeTrait for Text {
         let mut x = self.x.normalize(values, &params);
         let mut y = self.y.normalize(values, &params);
 
-        let chunks = self.make_chunks(node, acquired_nodes, cascaded, draw_ctx, x, y);
+        let chunks = self.make_chunks(node, acquired_nodes, cascaded, draw_ctx);
 
         let mut measured_chunks = Vec::new();
         for chunk in &chunks {
@@ -644,8 +636,12 @@ impl NodeTrait for Text {
 
         let mut positioned_chunks = Vec::new();
         for chunk in &measured_chunks {
-            let chunk_x = chunk.x.unwrap_or(x);
-            let chunk_y = chunk.y.unwrap_or(y);
+            let chunk_x = chunk
+                .x
+                .map_or_else(|| x, |l| l.normalize(&chunk.values, &params));
+            let chunk_y = chunk
+                .y
+                .map_or_else(|| y, |l| l.normalize(&chunk.values, &params));
 
             let positioned = PositionedChunk::from_measured(&chunk, draw_ctx, chunk_x, chunk_y);
 
@@ -678,7 +674,7 @@ pub struct TRef {
 impl TRef {
     fn to_chunks(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         chunks: &mut Vec<Chunk>,
@@ -691,7 +687,7 @@ impl TRef {
         let link = self.link.as_ref().unwrap();
         let values = cascaded.get();
 
-        if let Ok(acquired) = acquired_nodes.acquire(link, &[]) {
+        if let Ok(acquired) = acquired_nodes.acquire(link) {
             let c = acquired.get();
             extract_chars_children_to_chunks_recursively(chunks, &c, values, depth);
         } else {
@@ -706,23 +702,23 @@ impl TRef {
 
 fn extract_chars_children_to_chunks_recursively(
     chunks: &mut Vec<Chunk>,
-    node: &RsvgNode,
+    node: &Node,
     values: &ComputedValues,
     depth: usize,
 ) {
     for child in node.children() {
-        match child.borrow().get_type() {
-            NodeType::Chars => child
-                .borrow()
-                .get_impl::<NodeChars>()
-                .to_chunks(&child, values, chunks, 0.0, 0.0, depth),
-            _ => extract_chars_children_to_chunks_recursively(chunks, &child, values, depth + 1),
+        if child.is_chars() {
+            child
+                .borrow_chars()
+                .to_chunks(&child, values, chunks, None, None, depth)
+        } else {
+            extract_chars_children_to_chunks_recursively(chunks, &child, values, depth + 1)
         }
     }
 }
 
-impl NodeTrait for TRef {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for TRef {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!(xlink "href") => {
@@ -736,6 +732,8 @@ impl NodeTrait for TRef {
     }
 }
 
+impl Draw for TRef {}
+
 #[derive(Default)]
 pub struct TSpan {
     x: Option<Length<Horizontal>>,
@@ -747,27 +745,15 @@ pub struct TSpan {
 impl TSpan {
     fn to_chunks(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         chunks: &mut Vec<Chunk>,
-        dx: f64,
-        dy: f64,
         depth: usize,
     ) {
         let values = cascaded.get();
-
-        let params = draw_ctx.get_view_params();
-        let x = self.x.map(|l| l.normalize(&values, &params));
-        let y = self.y.map(|l| l.normalize(&values, &params));
-
-        let span_dx = dx + self.dx.map_or(0.0, |l| l.normalize(&values, &params));
-        let span_dy = dy + self.dy.map_or(0.0, |l| l.normalize(&values, &params));
-
-        if x.is_some() || y.is_some() {
-            chunks.push(Chunk::new(values, x, y));
-        }
+        chunks.push(Chunk::new(values, self.x, self.y));
 
         children_to_chunks(
             chunks,
@@ -775,15 +761,15 @@ impl TSpan {
             acquired_nodes,
             cascaded,
             draw_ctx,
-            span_dx,
-            span_dy,
+            self.dx,
+            self.dy,
             depth,
         );
     }
 }
 
-impl NodeTrait for TSpan {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for TSpan {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "x") => self.x = attr.parse(value).map(Some)?,
@@ -798,6 +784,8 @@ impl NodeTrait for TSpan {
     }
 }
 
+impl Draw for TSpan {}
+
 fn to_pango_units(v: f64) -> i32 {
     (v * f64::from(pango::SCALE) + 0.5) as i32
 }
@@ -858,7 +846,21 @@ impl From<FontStretch> for pango::Stretch {
 
 impl From<FontWeightSpec> for pango::Weight {
     fn from(w: FontWeightSpec) -> pango::Weight {
-        pango::Weight::__Unknown(w.numeric_weight().into())
+        match w {
+            FontWeightSpec::Normal => pango::Weight::Normal,
+            FontWeightSpec::Bold => pango::Weight::Bold,
+            FontWeightSpec::Bolder => pango::Weight::Ultrabold,
+            FontWeightSpec::Lighter => pango::Weight::Light,
+            FontWeightSpec::W100 => pango::Weight::Thin,
+            FontWeightSpec::W200 => pango::Weight::Ultralight,
+            FontWeightSpec::W300 => pango::Weight::Semilight,
+            FontWeightSpec::W400 => pango::Weight::Normal,
+            FontWeightSpec::W500 => pango::Weight::Medium,
+            FontWeightSpec::W600 => pango::Weight::Semibold,
+            FontWeightSpec::W700 => pango::Weight::Bold,
+            FontWeightSpec::W800 => pango::Weight::Ultrabold,
+            FontWeightSpec::W900 => pango::Weight::Heavy,
+        }
     }
 }
 
@@ -949,15 +951,15 @@ fn create_pango_layout(
     // See the construction of the XmlLang property
     // We use "" there as the default value; this means that the language is not set.
     // If the language *is* set, we can use it here.
-    if !values.xml_lang.0.is_empty() {
-        pango_context.set_language(&pango::Language::from(&values.xml_lang));
+    if !values.xml_lang().0.is_empty() {
+        pango_context.set_language(&pango::Language::from(&values.xml_lang()));
     }
 
-    pango_context.set_base_gravity(pango::Gravity::from(values.writing_mode));
+    pango_context.set_base_gravity(pango::Gravity::from(values.writing_mode()));
 
-    match (values.unicode_bidi, values.direction) {
+    match (values.unicode_bidi(), values.direction()) {
         (UnicodeBidi::Override, _) | (UnicodeBidi::Embed, _) => {
-            pango_context.set_base_dir(pango::Direction::from(values.direction));
+            pango_context.set_base_dir(pango::Direction::from(values.direction()));
         }
 
         (_, direction) if direction != Direction::Ltr => {
@@ -965,21 +967,21 @@ fn create_pango_layout(
         }
 
         (_, _) => {
-            pango_context.set_base_dir(pango::Direction::from(values.writing_mode));
+            pango_context.set_base_dir(pango::Direction::from(values.writing_mode()));
         }
     }
 
     let mut font_desc = pango_context.get_font_description().unwrap();
-    font_desc.set_family(&(values.font_family.0).0);
-    font_desc.set_style(pango::Style::from(values.font_style));
-    font_desc.set_variant(pango::Variant::from(values.font_variant));
-    font_desc.set_weight(pango::Weight::from(values.font_weight.0));
-    font_desc.set_stretch(pango::Stretch::from(values.font_stretch));
+    font_desc.set_family(&(values.font_family().0).0);
+    font_desc.set_style(pango::Style::from(values.font_style()));
+    font_desc.set_variant(pango::Variant::from(values.font_variant()));
+    font_desc.set_weight(pango::Weight::from(values.font_weight().0));
+    font_desc.set_stretch(pango::Stretch::from(values.font_stretch()));
 
     let params = draw_ctx.get_view_params();
 
     font_desc.set_size(to_pango_units(
-        values.font_size.0.normalize(values, &params),
+        values.font_size().0.normalize(values, &params),
     ));
 
     let layout = pango::Layout::new(&pango_context);
@@ -990,21 +992,21 @@ fn create_pango_layout(
 
     attr_list.insert(
         pango::Attribute::new_letter_spacing(to_pango_units(
-            values.letter_spacing.0.normalize(values, &params),
+            values.letter_spacing().0.normalize(values, &params),
         ))
         .unwrap(),
     );
 
-    if values.text_decoration.underline {
+    if values.text_decoration().underline {
         attr_list.insert(pango::Attribute::new_underline(pango::Underline::Single).unwrap());
     }
 
-    if values.text_decoration.strike {
+    if values.text_decoration().strike {
         attr_list.insert(pango::Attribute::new_strikethrough(true).unwrap());
     }
 
     layout.set_attributes(Some(&attr_list));
-    layout.set_alignment(pango::Alignment::from(values.direction));
+    layout.set_alignment(pango::Alignment::from(values.direction()));
     layout.set_text(text);
 
     layout
